@@ -11,11 +11,10 @@ import ipaddress
 import logging
 import os
 import re
-from typing import Any, Callable, Dict, List, Optional
-from urllib.parse import urlparse
+from typing import Callable, Dict, List, Optional, Tuple
 
 from cloudinit import subp, util
-from cloudinit.url_helper import UrlError, readurl
+from cloudinit.net.netops.iproute2 import Iproute2
 
 LOG = logging.getLogger(__name__)
 SYS_CLASS_NET = "/sys/class/net/"
@@ -73,7 +72,7 @@ def read_sys_net(
 ):
     dev_path = sys_dev_path(devname, path)
     try:
-        contents = util.load_file(dev_path)
+        contents = util.load_text_file(dev_path)
     except (OSError, IOError) as e:
         e_errno = getattr(e, "errno", None)
         if e_errno in (errno.ENOENT, errno.ENOTDIR):
@@ -167,6 +166,10 @@ def master_is_openvswitch(devname):
     return os.path.exists(ovs_path)
 
 
+def is_ib_interface(devname):
+    return read_sys_net_safe(devname, "type") == "32"
+
+
 @functools.lru_cache(maxsize=None)
 def openvswitch_is_installed() -> bool:
     """Return a bool indicating if Open vSwitch is installed in the system."""
@@ -238,7 +241,7 @@ def get_dev_features(devname):
 
 
 def has_netfail_standby_feature(devname):
-    """ Return True if VIRTIO_NET_F_STANDBY bit (62) is set.
+    """Return True if VIRTIO_NET_F_STANDBY bit (62) is set.
 
     https://github.com/torvalds/linux/blob/ \
         089cf7f6ecb266b6a4164919a2e69bd2f938374a/ \
@@ -395,65 +398,52 @@ def is_disabled_cfg(cfg):
     return cfg.get("config") == "disabled"
 
 
-def find_candidate_nics(
-    blacklist_drivers: Optional[List[str]] = None,
-) -> List[str]:
+def find_candidate_nics() -> List[str]:
     """Get the list of network interfaces viable for networking.
 
     @return List of interfaces, sorted naturally.
     """
     if util.is_FreeBSD() or util.is_DragonFlyBSD():
-        return find_candidate_nics_on_freebsd(blacklist_drivers)
+        return find_candidate_nics_on_freebsd()
     elif util.is_NetBSD() or util.is_OpenBSD():
-        return find_candidate_nics_on_netbsd_or_openbsd(blacklist_drivers)
+        return find_candidate_nics_on_netbsd_or_openbsd()
     else:
-        return find_candidate_nics_on_linux(blacklist_drivers)
+        return find_candidate_nics_on_linux()
 
 
-def find_fallback_nic(
-    blacklist_drivers: Optional[List[str]] = None,
-) -> Optional[str]:
+def find_fallback_nic() -> Optional[str]:
     """Get the name of the 'fallback' network device."""
     if util.is_FreeBSD() or util.is_DragonFlyBSD():
-        return find_fallback_nic_on_freebsd(blacklist_drivers)
+        return find_fallback_nic_on_freebsd()
     elif util.is_NetBSD() or util.is_OpenBSD():
-        return find_fallback_nic_on_netbsd_or_openbsd(blacklist_drivers)
+        return find_fallback_nic_on_netbsd_or_openbsd()
     else:
-        return find_fallback_nic_on_linux(blacklist_drivers)
+        return find_fallback_nic_on_linux()
 
 
-def find_candidate_nics_on_netbsd_or_openbsd(
-    blacklist_drivers: Optional[List[str]] = None,
-) -> List[str]:
+def find_candidate_nics_on_netbsd_or_openbsd() -> List[str]:
     """Get the names of the candidate network devices on NetBSD/OpenBSD.
 
-    @param blacklist_drivers: currently ignored
     @return list of sorted interfaces
     """
     return sorted(get_interfaces_by_mac().values(), key=natural_sort_key)
 
 
-def find_fallback_nic_on_netbsd_or_openbsd(
-    blacklist_drivers: Optional[List[str]] = None,
-) -> Optional[str]:
+def find_fallback_nic_on_netbsd_or_openbsd() -> Optional[str]:
     """Get the 'fallback' network device name on NetBSD/OpenBSD.
 
-    @param blacklist_drivers: currently ignored
     @return default interface, or None
     """
-    names = find_candidate_nics_on_netbsd_or_openbsd(blacklist_drivers)
+    names = find_candidate_nics_on_netbsd_or_openbsd()
     if names:
         return names[0]
 
     return None
 
 
-def find_candidate_nics_on_freebsd(
-    blacklist_drivers: Optional[List[str]] = None,
-) -> List[str]:
+def find_candidate_nics_on_freebsd() -> List[str]:
     """Get the names of the candidate network devices on FreeBSD.
 
-    @param blacklist_drivers: Currently ignored.
     @return List of sorted interfaces.
     """
     stdout, _stderr = subp.subp(["ifconfig", "-l", "-u", "ether"])
@@ -466,32 +456,23 @@ def find_candidate_nics_on_freebsd(
     return sorted(get_interfaces_by_mac().values(), key=natural_sort_key)
 
 
-def find_fallback_nic_on_freebsd(
-    blacklist_drivers: Optional[List[str]] = None,
-) -> Optional[str]:
+def find_fallback_nic_on_freebsd() -> Optional[str]:
     """Get the 'fallback' network device name on FreeBSD.
 
-    @param blacklist_drivers: Currently ignored.
     @return List of sorted interfaces.
     """
-    names = find_candidate_nics_on_freebsd(blacklist_drivers)
+    names = find_candidate_nics_on_freebsd()
     if names:
         return names[0]
 
     return None
 
 
-def find_candidate_nics_on_linux(
-    blacklist_drivers: Optional[List[str]] = None,
-) -> List[str]:
+def find_candidate_nics_on_linux() -> List[str]:
     """Get the names of the candidate network devices on Linux.
 
-    @param blacklist_drivers: Filter out NICs with these drivers.
     @return List of sorted interfaces.
     """
-    if not blacklist_drivers:
-        blacklist_drivers = []
-
     if "net.ifnames=0" in util.get_cmdline():
         LOG.debug("Stable ifnames disabled by net.ifnames=0 in /proc/cmdline")
     else:
@@ -505,37 +486,25 @@ def find_candidate_nics_on_linux(
                 "Found unstable nic names: %s; calling udevadm settle",
                 unstable,
             )
-            msg = "Waiting for udev events to settle"
-            util.log_time(LOG.debug, msg, func=util.udevadm_settle)
+            util.udevadm_settle()
 
     # sort into interfaces with carrier, interfaces which could have carrier,
     # and ignore interfaces that are definitely disconnected
     connected = []
     possibly_connected = []
-    for interface in get_devicelist():
+    for interface, _, _, _ in get_interfaces(
+        filter_openvswitch_internal=False,
+        filter_slave_if_master_not_bridge_bond_openvswitch=False,
+        filter_vlan=False,
+        filter_without_own_mac=False,
+        filter_zero_mac=False,
+        log_filtered_reasons=True,
+    ):
         if interface == "lo":
             continue
-        driver = device_driver(interface)
-        if driver in blacklist_drivers:
-            LOG.debug(
-                "Ignoring interface with %s driver: %s", driver, interface
-            )
-            continue
-        if not read_sys_net_safe(interface, "address"):
-            LOG.debug("Ignoring interface without mac: %s", interface)
-            continue
         if interface.startswith("veth"):
             LOG.debug("Ignoring veth interface: %s", interface)
             continue
-        if is_bridge(interface):
-            LOG.debug("Ignoring bridge interface: %s", interface)
-            continue
-        if is_bond(interface):
-            LOG.debug("Ignoring bond interface: %s", interface)
-            continue
-        if is_netfailover(interface):
-            LOG.debug("Ignoring failover interface: %s", interface)
-            continue
         carrier = read_sys_net_int(interface, "carrier")
         if carrier:
             connected.append(interface)
@@ -571,27 +540,22 @@ def find_candidate_nics_on_linux(
     return sorted_interfaces
 
 
-def find_fallback_nic_on_linux(
-    blacklist_drivers: Optional[List[str]] = None,
-) -> Optional[str]:
+def find_fallback_nic_on_linux() -> Optional[str]:
     """Get the 'fallback' network device name on Linux.
 
-    @param blacklist_drivers: Ignore devices with these drivers.
     @return List of sorted interfaces.
     """
-    names = find_candidate_nics_on_linux(blacklist_drivers)
+    names = find_candidate_nics_on_linux()
     if names:
         return names[0]
 
     return None
 
 
-def generate_fallback_config(blacklist_drivers=None, config_driver=None):
+def generate_fallback_config(config_driver=None) -> Optional[dict]:
     """Generate network cfg v2 for dhcp on the NIC most likely connected."""
-    if not config_driver:
-        config_driver = False
 
-    target_name = find_fallback_nic(blacklist_drivers=blacklist_drivers)
+    target_name = find_fallback_nic()
     if not target_name:
         # can't read any interfaces addresses (or there are none); give up
         return None
@@ -603,11 +567,16 @@ def generate_fallback_config(blacklist_drivers=None, config_driver=None):
         match = {
             "macaddress": read_sys_net_safe(target_name, "address").lower()
         }
-    cfg = {"dhcp4": True, "set-name": target_name, "match": match}
     if config_driver:
         driver = device_driver(target_name)
         if driver:
-            cfg["match"]["driver"] = driver
+            match["driver"] = driver
+    cfg = {
+        "dhcp4": True,
+        "dhcp6": True,
+        "set-name": target_name,
+        "match": match,
+    }
     nconf = {"ethernets": {target_name: cfg}, "version": 2}
     return nconf
 
@@ -668,7 +637,7 @@ def interface_has_own_mac(ifname, strict=False):
     are bonds or vlans that inherit their mac from another device.
     Possible values are:
       0: permanent address    2: stolen from another device
-      1: randomly generated   3: set using dev_set_mac_address"""
+    1: randomly generated   3: set using dev_set_mac_address"""
 
     assign_type = read_sys_net_int(ifname, "addr_assign_type")
     if assign_type is None:
@@ -696,7 +665,7 @@ def _get_current_rename_info(check_downable=True):
          }}
     """
     cur_info = {}
-    for (name, mac, driver, device_id) in get_interfaces():
+    for name, mac, driver, device_id in get_interfaces():
         cur_info[name] = {
             "downable": None,
             "device_id": device_id,
@@ -748,16 +717,7 @@ def _rename_interfaces(
     LOG.debug("Detected interfaces %s", cur_info)
 
     def update_byname(bymac):
-        return dict((data["name"], data) for data in cur_info.values())
-
-    def rename(cur, new):
-        subp.subp(["ip", "link", "set", cur, "name", new], capture=True)
-
-    def down(name):
-        subp.subp(["ip", "link", "set", name, "down"], capture=True)
-
-    def up(name):
-        subp.subp(["ip", "link", "set", name, "up"], capture=True)
+        return dict((data["name"], data) for data in bymac.values())
 
     ops = []
     errors = []
@@ -802,7 +762,7 @@ def _rename_interfaces(
     for mac, new_name, driver, device_id in renames:
         if mac:
             mac = mac.lower()
-        cur_ops = []
+        cur_ops: list = []
         cur = find_entry(mac, driver, device_id)
         if not cur:
             if strict_present:
@@ -846,7 +806,7 @@ def _rename_interfaces(
                 else:
                     cur_ops.append(("down", mac, new_name, (new_name,)))
 
-            tmp_name = None
+            tmp_name: Optional[str] = None
             while tmp_name is None or tmp_name in cur_byname:
                 tmpi += 1
                 tmp_name = tmpname_fmt % tmpi
@@ -862,19 +822,27 @@ def _rename_interfaces(
         cur_byname = update_byname(cur_info)
         ops += cur_ops
 
-    opmap = {"rename": rename, "down": down, "up": up}
+    opmap: Dict[str, Callable] = {
+        "rename": Iproute2.link_rename,
+        "down": Iproute2.link_down,
+        "up": Iproute2.link_up,
+    }
 
-    if len(ops) + len(ups) == 0:
+    if not (ops) and not (ups):
         if len(errors):
-            LOG.debug("unable to do any work for renaming of %s", renames)
+            LOG.warning(
+                "Unable to rename interfaces: %s due to errors: %s",
+                renames,
+                errors,
+            )
         else:
             LOG.debug("no work necessary for renaming of %s", renames)
     else:
-        LOG.debug("achieving renaming of %s with ops %s", renames, ops + ups)
+        LOG.debug("Renamed %s with ops %s", renames, ops + ups)
 
         for op, mac, new_name, params in ops + ups:
             try:
-                opmap.get(op)(*params)
+                opmap[op](*params)
             except Exception as e:
                 errors.append(
                     "[unknown] Error performing %s%s for %s, %s: %s"
@@ -882,7 +850,7 @@ def _rename_interfaces(
                 )
 
     if len(errors):
-        raise Exception("\n".join(errors))
+        raise RuntimeError("\n".join(errors))
 
 
 def get_interface_mac(ifname):
@@ -909,23 +877,15 @@ def get_ib_interface_hwaddr(ifname, ethernet_format):
         return mac
 
 
-def get_interfaces_by_mac(blacklist_drivers=None) -> dict:
+def get_interfaces_by_mac() -> dict:
     if util.is_FreeBSD() or util.is_DragonFlyBSD():
-        return get_interfaces_by_mac_on_freebsd(
-            blacklist_drivers=blacklist_drivers
-        )
+        return get_interfaces_by_mac_on_freebsd()
     elif util.is_NetBSD():
-        return get_interfaces_by_mac_on_netbsd(
-            blacklist_drivers=blacklist_drivers
-        )
+        return get_interfaces_by_mac_on_netbsd()
     elif util.is_OpenBSD():
-        return get_interfaces_by_mac_on_openbsd(
-            blacklist_drivers=blacklist_drivers
-        )
+        return get_interfaces_by_mac_on_openbsd()
     else:
-        return get_interfaces_by_mac_on_linux(
-            blacklist_drivers=blacklist_drivers
-        )
+        return get_interfaces_by_mac_on_linux()
 
 
 def find_interface_name_from_mac(mac: str) -> Optional[str]:
@@ -935,7 +895,7 @@ def find_interface_name_from_mac(mac: str) -> Optional[str]:
     return None
 
 
-def get_interfaces_by_mac_on_freebsd(blacklist_drivers=None) -> dict:
+def get_interfaces_by_mac_on_freebsd() -> dict:
     (out, _) = subp.subp(["ifconfig", "-a", "ether"])
 
     # flatten each interface block in a single line
@@ -963,7 +923,7 @@ def get_interfaces_by_mac_on_freebsd(blacklist_drivers=None) -> dict:
     return results
 
 
-def get_interfaces_by_mac_on_netbsd(blacklist_drivers=None) -> dict:
+def get_interfaces_by_mac_on_netbsd() -> dict:
     ret = {}
     re_field_match = (
         r"(?P<ifname>\w+).*address:\s"
@@ -979,7 +939,7 @@ def get_interfaces_by_mac_on_netbsd(blacklist_drivers=None) -> dict:
     return ret
 
 
-def get_interfaces_by_mac_on_openbsd(blacklist_drivers=None) -> dict:
+def get_interfaces_by_mac_on_openbsd() -> dict:
     ret = {}
     re_field_match = (
         r"(?P<ifname>\w+).*lladdr\s"
@@ -995,52 +955,21 @@ def get_interfaces_by_mac_on_openbsd(blacklist_drivers=None) -> dict:
     return ret
 
 
-def get_interfaces_by_mac_on_linux(blacklist_drivers=None) -> dict:
+def get_interfaces_by_mac_on_linux() -> dict:
     """Build a dictionary of tuples {mac: name}.
 
     Bridges and any devices that have a 'stolen' mac are excluded."""
     ret: dict = {}
-    driver_map: dict = {}
-    for name, mac, driver, _devid in get_interfaces(
-        blacklist_drivers=blacklist_drivers
-    ):
-        if mac in ret:
-            raise_duplicate_mac_error = True
-            msg = "duplicate mac found! both '%s' and '%s' have mac '%s'." % (
-                name,
-                ret[mac],
-                mac,
-            )
-            # Hyper-V netvsc driver will register a VF with the same mac
-            #
-            # The VF will be enslaved to the master nic shortly after
-            # registration. If cloud-init starts enumerating the interfaces
-            # before the completion of the enslaving process, it will see
-            # two different nics with duplicate mac. Cloud-init should ignore
-            # the slave nic (which does not have hv_netvsc driver).
-            if driver != driver_map[mac]:
-                if driver_map[mac] == "hv_netvsc":
-                    LOG.warning(
-                        msg + " Ignoring '%s' due to driver '%s' and "
-                        "'%s' having driver hv_netvsc."
-                        % (name, driver, ret[mac])
-                    )
-                    continue
-                if driver == "hv_netvsc":
-                    raise_duplicate_mac_error = False
-                    LOG.warning(
-                        msg + " Ignoring '%s' due to driver '%s' and "
-                        "'%s' having driver hv_netvsc."
-                        % (ret[mac], driver_map[mac], name)
-                    )
 
+    for name, mac, driver, _devid in get_interfaces():
+        if mac in ret:
             # This is intended to be a short-term fix of LP: #1997922
             # Long term, we should better handle configuration of virtual
             # devices where duplicate MACs are expected early in boot if
             # cloud-init happens to enumerate network interfaces before drivers
             # have fully initialized the leader/subordinate relationships for
             # those devices or switches.
-            if driver == "mscc_felix" or driver == "fsl_enetc":
+            if driver in ("fsl_enetc", "mscc_felix", "qmi_wwan"):
                 LOG.debug(
                     "Ignoring duplicate macs from '%s' and '%s' due to "
                     "driver '%s'.",
@@ -1050,11 +979,14 @@ def get_interfaces_by_mac_on_linux(blacklist_drivers=None) -> dict:
                 )
                 continue
 
-            if raise_duplicate_mac_error:
-                raise RuntimeError(msg)
+            msg = "duplicate mac found! both '%s' and '%s' have mac '%s'." % (
+                name,
+                ret[mac],
+                mac,
+            )
+            raise RuntimeError(msg)
 
         ret[mac] = name
-        driver_map[mac] = driver
 
         # Pretend that an Infiniband GUID is an ethernet address for Openstack
         # configuration purposes
@@ -1094,53 +1026,112 @@ def get_interfaces_by_mac_on_linux(blacklist_drivers=None) -> dict:
     return ret
 
 
-def get_interfaces(blacklist_drivers=None) -> list:
+def get_interfaces(
+    filter_hyperv_vf_with_synthetic: bool = True,
+    filter_openvswitch_internal: bool = True,
+    filter_slave_if_master_not_bridge_bond_openvswitch: bool = True,
+    filter_vlan: bool = True,
+    filter_without_own_mac: bool = True,
+    filter_zero_mac: bool = True,
+    log_filtered_reasons: bool = False,
+) -> list:
     """Return list of interface tuples (name, mac, driver, device_id)
 
     Bridges and any devices that have a 'stolen' mac are excluded."""
+    filtered_logger = LOG.debug if log_filtered_reasons else lambda *args: None
     ret = []
-    if blacklist_drivers is None:
-        blacklist_drivers = []
     devs = get_devicelist()
     # 16 somewhat arbitrarily chosen.  Normally a mac is 6 '00:' tokens.
     zero_mac = ":".join(("00",) * 16)
     for name in devs:
-        if not interface_has_own_mac(name):
+        if filter_without_own_mac and not interface_has_own_mac(name):
             continue
         if is_bridge(name):
+            filtered_logger("Ignoring bridge interface: %s", name)
             continue
-        if is_vlan(name):
+        if filter_vlan and is_vlan(name):
             continue
         if is_bond(name):
+            filtered_logger("Ignoring bond interface: %s", name)
+            continue
+        if (
+            filter_slave_if_master_not_bridge_bond_openvswitch
+            and get_master(name) is not None
+            and not master_is_bridge_or_bond(name)
+            and not master_is_openvswitch(name)
+        ):
             continue
-        if get_master(name) is not None:
-            if not master_is_bridge_or_bond(
-                name
-            ) and not master_is_openvswitch(name):
-                continue
         if is_netfailover(name):
+            filtered_logger("Ignoring failover interface: %s", name)
             continue
         mac = get_interface_mac(name)
         # some devices may not have a mac (tun0)
         if not mac:
+            filtered_logger("Ignoring interface without mac: %s", name)
             continue
         # skip nics that have no mac (00:00....)
-        if name != "lo" and mac == zero_mac[: len(mac)]:
+        if filter_zero_mac and name != "lo" and mac == zero_mac[: len(mac)]:
             continue
-        if is_openvswitch_internal_interface(name):
+        if filter_openvswitch_internal and is_openvswitch_internal_interface(
+            name
+        ):
             continue
-        # skip nics that have drivers blacklisted
         driver = device_driver(name)
-        if driver in blacklist_drivers:
-            continue
         ret.append((name, mac, driver, device_devid(name)))
+
+    # Last-pass filter(s) which need the full device list to perform properly.
+    if filter_hyperv_vf_with_synthetic:
+        filter_hyperv_vf_with_synthetic_interface(filtered_logger, ret)
+
     return ret
 
 
+def filter_hyperv_vf_with_synthetic_interface(
+    filtered_logger: Callable[..., None],
+    interfaces: List[Tuple[str, str, str, str]],
+) -> None:
+    """Filter Hyper-V SR-IOV/VFs when used with synthetic hv_netvsc.
+
+    Hyper-V's netvsc driver may register an SR-IOV/VF interface with a mac
+    that matches the synthetic (hv_netvsc) interface.  This VF will be
+    enslaved to the synthetic interface, but cloud-init may be racing this
+    process.  The [perhaps-yet-to-be-enslaved] VF should never be directly
+    configured, so we filter interfaces that duplicate any hv_netvsc mac
+    address, as this the most reliable indicator that it is meant to be
+    subordinate to the synthetic interface.
+
+    VF drivers will be mlx4_core, mlx5_core, or mana.  However, given that
+    this list of drivers has changed over time and mana's dependency on
+    hv_netvsc is expected to be removed in the future, we no longer rely
+    on these names. Note that this will not affect mlx4/5 instances outside
+    of Hyper-V, as it only affects environments where hv_netvsc is present.
+    """
+    hv_netvsc_mac_to_name = {
+        i[1]: i[0] for i in interfaces if i[2] == "hv_netvsc"
+    }
+    interfaces_to_remove = [
+        i
+        for i in interfaces
+        if i[1] in hv_netvsc_mac_to_name and i[2] != "hv_netvsc"
+    ]
+
+    for interface in interfaces_to_remove:
+        name, mac, driver, _ = interface
+        filtered_logger(
+            "Ignoring %r VF interface with driver %r due to "
+            "synthetic hv_netvsc interface %r with mac address %r.",
+            name,
+            driver,
+            hv_netvsc_mac_to_name[mac],
+            mac,
+        )
+        interfaces.remove(interface)
+
+
 def get_ib_hwaddrs_by_interface():
     """Build a dictionary mapping Infiniband interface names to their hardware
     address."""
-    ret = {}
+    ret: Dict[str, str] = {}
     for name, _, _, _ in get_interfaces():
         ib_mac = get_ib_interface_hwaddr(name, False)
         if ib_mac:
@@ -1153,45 +1144,6 @@ def get_ib_hwaddrs_by_interface():
     return ret
 
 
-def has_url_connectivity(url_data: Dict[str, Any]) -> bool:
-    """Return true when the instance has access to the provided URL.
-
-    Logs a warning if url is not the expected format.
-
-    url_data is a dictionary of kwargs to send to readurl. E.g.:
-
-    has_url_connectivity({
-        "url": "http://example.invalid",
-        "headers": {"some": "header"},
-        "timeout": 10
-    })
-    """
-    if "url" not in url_data:
-        LOG.warning(
-            "Ignoring connectivity check. No 'url' to check in %s", url_data
-        )
-        return False
-    url = url_data["url"]
-    try:
-        result = urlparse(url)
-        if not any([result.scheme == "http", result.scheme == "https"]):
-            LOG.warning(
-                "Ignoring connectivity check. Invalid URL scheme %s",
-                url.scheme,
-            )
-            return False
-    except ValueError as err:
-        LOG.warning("Ignoring connectivity check. Invalid URL %s", err)
-        return False
-    if "timeout" not in url_data:
-        url_data["timeout"] = 5
-    try:
-        readurl(**url_data)
-    except UrlError:
-        return False
-    return True
-
-
 def maybe_get_address(convert_to_address: Callable, address: str, **kwargs):
     """Use a function to return an address. If conversion throws a ValueError
     exception return False.
@@ -1287,10 +1239,54 @@ def is_ipv6_network(address: str) -> bool:
     )
 
 
+def is_ip_in_subnet(address: str, subnet: str) -> bool:
+    """Returns a bool indicating if ``s`` is in subnet.
+
+    :param address:
+        The string of IP address.
+
+    :param subnet:
+        The string of subnet.
+
+    :return:
+        A bool indicating if the string is in subnet.
+    """
+    ip_address = ipaddress.ip_address(address)
+    subnet_network = ipaddress.ip_network(subnet, strict=False)
+    return ip_address in subnet_network
+
+
+def should_add_gateway_onlink_flag(gateway: str, subnet: str) -> bool:
+    """Returns a bool indicating if should add gateway onlink flag.
+
+    :param gateway:
+        The string of gateway address.
+
+    :param subnet:
+        The string of subnet.
+
+    :return:
+        A bool indicating if the string is in subnet.
+    """
+    try:
+        return not is_ip_in_subnet(gateway, subnet)
+    except ValueError as e:
+        LOG.warning(
+            "Failed to check whether gateway %s"
+            " is contained within subnet %s: %s",
+            gateway,
+            subnet,
+            e,
+        )
+        return False
+
+
 def subnet_is_ipv6(subnet) -> bool:
     """Common helper for checking network_state subnets for ipv6."""
     # 'static6', 'dhcp6', 'ipv6_dhcpv6-stateful', 'ipv6_dhcpv6-stateless' or
     # 'ipv6_slaac'
+    # This function is inappropriate for v2-based routes as routes defined
+    # under v2 subnets can contain ipv4 and ipv6 simultaneously
     if subnet["type"].endswith("6") or subnet["type"] in IPV6_DYNAMIC_TYPES:
         # This is a request either static6 type or DHCPv6.
         return True
