@@ -20,7 +20,7 @@ from tests.integration_tests.integration_settings import PLATFORM
     ],
 )
 def test_ephemeral(
-    instance_type, is_ephemeral, session_cloud: IntegrationCloud, setup_image
+    instance_type, is_ephemeral, session_cloud: IntegrationCloud
 ):
     if is_ephemeral:
         expected_log = (
