@@ -20,6 +20,14 @@ runcmd:
   - echo 'hi' > /var/tmp/test
 """
 
+FAILING_USER_DATA = """\
+#cloud-config
+bootcmd:
+  - exit 1
+runcmd:
+  - exit 1
+"""
+
 # The '-' in 'hashed-password' fails schema validation
 INVALID_USER_DATA_SCHEMA = """\
 #cloud-config
@@ -36,19 +44,26 @@ users:
 
 
 @pytest.mark.user_data(VALID_USER_DATA)
-def test_valid_userdata(client: IntegrationInstance):
-    """Test `cloud-init schema` with valid userdata.
+class TestValidUserData:
+    def test_schema_status(self, class_client: IntegrationInstance):
+        """Test `cloud-init schema` with valid userdata.
 
-    PR #575
-    """
-    result = client.execute("cloud-init schema --system")
-    assert result.ok
-    assert "Valid schema user-data" in result.stdout.strip()
-    result = client.execute("cloud-init status --long")
-    assert 0 == result.return_code, (
-        f"Unexpected exit {result.return_code} from cloud-init status:"
-        f" {result}"
-    )
+        PR #575
+        """
+        result = class_client.execute("cloud-init schema --system")
+        assert result.ok
+        assert "Valid schema user-data" in result.stdout.strip()
+        result = class_client.execute("cloud-init status --long")
+        assert 0 == result.return_code, (
+            f"Unexpected exit {result.return_code} from cloud-init status:"
+            f" {result}"
+        )
+
+    def test_modules_init(self, class_client: IntegrationInstance):
+        for mode in ("init", "config", "final"):
+            result = class_client.execute(f"cloud-init modules --mode {mode}")
+            assert result.ok
+            assert f"'modules:{mode}'" in result.stdout.strip()
 
 
 @pytest.mark.skipif(
@@ -93,8 +108,27 @@ def test_invalid_userdata_schema(client: IntegrationInstance):
     ), f"Unexpected exit code {result.return_code}"
     log = client.read_from_file("/var/log/cloud-init.log")
     warning = (
-        "[WARNING]: Invalid cloud-config provided: Please run "
-        "'sudo cloud-init schema --system' to see the schema errors."
+        "[WARNING]: cloud-config failed schema validation! "
+        "You may run 'sudo cloud-init schema --system' to check the details."
     )
     assert warning in log
     assert "asdfasdf" not in log
+
+
+@pytest.mark.user_data(FAILING_USER_DATA)
+def test_failing_userdata_modules_exit_codes(client: IntegrationInstance):
+    """Test failing in modules representd in exit status.
+
+    To ensure we don't miss any errors or warnings if a service happens
+    to be restarted, any further module invocations will exit with error
+    on the same boot if a previous invocation exited with error.
+
+    In this test, both bootcmd and runcmd will exit with error the first time.
+    The second time, runcmd will run cleanly, but still exit with error.
+    Since bootcmd runs in init timeframe, and runcmd runs in final timeframe,
+    expect error from those two modes.
+    """
+    for mode in ("init", "config", "final"):
+        result = client.execute(f"cloud-init modules --mode {mode}")
+        assert result.ok if mode == "config" else result.failed
+        assert f"'modules:{mode}'" in result.stdout.strip()
