@@ -5,6 +5,7 @@ import pytest
 from paramiko.ssh_exception import SSHException
 
 from tests.integration_tests.instances import IntegrationInstance
+from tests.integration_tests.integration_settings import PLATFORM
 from tests.integration_tests.releases import CURRENT_RELEASE, IS_UBUNTU
 from tests.integration_tests.util import get_test_rsa_keypair
 
@@ -128,6 +129,10 @@ DEFAULT_KEYS_USERDATA = _USERDATA.format(bootcmd='""')
 @pytest.mark.skipif(
     not IS_UBUNTU, reason="Tests permissions specific to Ubuntu releases"
 )
+@pytest.mark.skipif(
+    PLATFORM == "qemu",
+    reason="QEMU cloud manually adding key interferes with test",
+)
 @pytest.mark.user_data(DEFAULT_KEYS_USERDATA)
 def test_authorized_keys_default(client: IntegrationInstance):
     expected_keys = [
@@ -159,6 +164,10 @@ AUTHORIZED_KEYS2_USERDATA = _USERDATA.format(
 @pytest.mark.skipif(
     not IS_UBUNTU, reason="Tests permissions specific to Ubuntu releases"
 )
+@pytest.mark.skipif(
+    PLATFORM == "qemu",
+    reason="QEMU cloud manually adding key interferes with test",
+)
 @pytest.mark.user_data(AUTHORIZED_KEYS2_USERDATA)
 def test_authorized_keys2(client: IntegrationInstance):
     expected_keys = [
@@ -190,6 +199,10 @@ NESTED_KEYS_USERDATA = _USERDATA.format(
 @pytest.mark.skipif(
     not IS_UBUNTU, reason="Tests permissions specific to Ubuntu releases"
 )
+@pytest.mark.skipif(
+    PLATFORM == "qemu",
+    reason="QEMU cloud manually adding key interferes with test",
+)
 @pytest.mark.user_data(NESTED_KEYS_USERDATA)
 def test_nested_keys(client: IntegrationInstance):
     expected_keys = [
@@ -213,6 +226,10 @@ EXTERNAL_KEYS_USERDATA = _USERDATA.format(
 @pytest.mark.skipif(
     not IS_UBUNTU, reason="Tests permissions specific to Ubuntu releases"
 )
+@pytest.mark.skipif(
+    PLATFORM == "qemu",
+    reason="QEMU cloud manually adding key interferes with test",
+)
 @pytest.mark.user_data(EXTERNAL_KEYS_USERDATA)
 def test_external_keys(client: IntegrationInstance):
     expected_keys = [
