@@ -3,6 +3,7 @@ import logging
 
 import pytest
 
+from cloudinit import subp
 from cloudinit.config import cc_landscape
 from cloudinit.config.schema import (
     SchemaValidationError,
@@ -14,8 +15,10 @@ from tests.unittests.util import get_cloud
 
 LOG = logging.getLogger(__name__)
 
+MPATH = "cloudinit.config.cc_landscape"
 
-@mock.patch("cloudinit.config.cc_landscape.subp.subp")
+
+@mock.patch(f"{MPATH}.subp.subp")
 class TestLandscape:
     def test_skip_empty_landscape_cloudconfig(self, m_subp):
         """Empty landscape cloud-config section does no work."""
@@ -40,14 +43,12 @@ class TestLandscape:
         mycloud = get_cloud("ubuntu")
         mycloud.distro = mock.MagicMock()
         cfg = {"landscape": {"client": {}}}
-        default_fn = tmpdir.join("default")
         wrap_and_call(
             "cloudinit.config.cc_landscape",
             {
                 "LSC_CLIENT_CFG_FILE": {
                     "new": tmpdir.join("client.conf").strpath
                 },
-                "LS_DEFAULT_FILE": {"new": default_fn.strpath},
             },
             cc_landscape.handle,
             "notimportant",
@@ -59,6 +60,9 @@ class TestLandscape:
             ["landscape-client"]
         )
         assert [
+            mock.call(
+                ["landscape-config", "--silent", "--is-registered"], rcs=[5]
+            ),
             mock.call(
                 [
                     "landscape-config",
@@ -81,7 +85,6 @@ class TestLandscape:
         """Call landscape-config with any filesystem overrides."""
         mycloud = get_cloud("ubuntu")
         mycloud.distro = mock.MagicMock()
-        default_fn = tmpdir.join("default")
         client_fn = tmpdir.join("client.conf")
         client_fn.write("[client]\ndata_path = /var/lib/data\n")
         cfg = {
@@ -91,6 +94,9 @@ class TestLandscape:
             }
         }
         expected_calls = [
+            mock.call(
+                ["landscape-config", "--silent", "--is-registered"], rcs=[5]
+            ),
             mock.call(
                 [
                     "landscape-config",
@@ -110,7 +116,6 @@ class TestLandscape:
             "cloudinit.config.cc_landscape",
             {
                 "LSC_CLIENT_CFG_FILE": {"new": client_fn.strpath},
-                "LS_DEFAULT_FILE": {"new": default_fn.strpath},
             },
             cc_landscape.handle,
             "notimportant",
@@ -122,7 +127,6 @@ class TestLandscape:
             ["landscape-client"]
         )
         assert expected_calls == m_subp.call_args_list
-        assert "RUN=1\n" == default_fn.read()
 
     def test_handler_writes_merged_client_config_file_with_defaults(
         self, m_subp, tmpdir
@@ -131,11 +135,13 @@ class TestLandscape:
         # Write existing sparse client.conf file
         client_fn = tmpdir.join("client.conf")
         client_fn.write("[client]\ncomputer_title = My PC\n")
-        default_fn = tmpdir.join("default")
         mycloud = get_cloud("ubuntu")
         mycloud.distro = mock.MagicMock()
         cfg = {"landscape": {"client": {}}}
         expected_calls = [
+            mock.call(
+                ["landscape-config", "--silent", "--is-registered"], rcs=[5]
+            ),
             mock.call(
                 [
                     "landscape-config",
@@ -157,7 +163,6 @@ class TestLandscape:
             "cloudinit.config.cc_landscape",
             {
                 "LSC_CLIENT_CFG_FILE": {"new": client_fn.strpath},
-                "LS_DEFAULT_FILE": {"new": default_fn.strpath},
             },
             cc_landscape.handle,
             "notimportant",
@@ -174,11 +179,13 @@ class TestLandscape:
         # Write empty sparse client.conf file
         client_fn = tmpdir.join("client.conf")
         client_fn.write("")
-        default_fn = tmpdir.join("default")
         mycloud = get_cloud("ubuntu")
         mycloud.distro = mock.MagicMock()
         cfg = {"landscape": {"client": {"computer_title": 'My" PC'}}}
         expected_calls = [
+            mock.call(
+                ["landscape-config", "--silent", "--is-registered"], rcs=[5]
+            ),
             mock.call(
                 [
                     "landscape-config",
@@ -200,7 +207,6 @@ class TestLandscape:
             "cloudinit.config.cc_landscape",
             {
                 "LSC_CLIENT_CFG_FILE": {"new": client_fn.strpath},
-                "LS_DEFAULT_FILE": {"new": default_fn.strpath},
             },
             cc_landscape.handle,
             "notimportant",
@@ -210,6 +216,37 @@ class TestLandscape:
         )
         assert expected_calls == m_subp.call_args_list
 
+    @mock.patch(f"{MPATH}.merge_together")
+    def test_handler_client_failed_registering(self, m_merge_together, m_subp):
+        """landscape-client could not be registered"""
+        mycloud = get_cloud("ubuntu")
+        mycloud.distro = mock.MagicMock()
+        cfg = {"landscape": {"client": {"computer_title": 'My" PC'}}}
+        m_subp.side_effect = subp.ProcessExecutionError(
+            "Could not register client"
+        )
+        match = (
+            "Failure registering client:\nUnexpected error while"
+            " running command.\nCommand: -\nExit code: -\nReason: -\n"
+            "Stdout: Could not register client\nStderr: -"
+        )
+        with pytest.raises(RuntimeError, match=match):
+            cc_landscape.handle("notimportant", cfg, mycloud, None)
+
+    @mock.patch(f"{MPATH}.merge_together")
+    def test_handler_client_is_already_registered(
+        self, m_merge_together, m_subp, caplog
+    ):
+        """landscape-client is already registered"""
+        mycloud = get_cloud("ubuntu")
+        mycloud.distro = mock.MagicMock()
+        cfg = {"landscape": {"client": {"computer_title": 'My" PC'}}}
+        m_subp.side_effect = subp.ProcessExecutionError(
+            "Client already registered to Landscape", exit_code=0
+        )
+        cc_landscape.handle("notimportant", cfg, mycloud, None)
+        assert "Client already registered to Landscape" in caplog.text
+
 
 class TestLandscapeSchema:
     @pytest.mark.parametrize(
