@@ -3,24 +3,19 @@ import copy
 import os
 import re
 import shutil
-from functools import partial
 from os.path import dirname
+from typing import Any, Dict, List
 
 import pytest
 
-from cloudinit import helpers, util
+from cloudinit import util
 from cloudinit.config import cc_ntp
 from cloudinit.config.schema import (
     SchemaValidationError,
     get_schema,
     validate_cloudconfig_schema,
 )
-from tests.unittests.helpers import (
-    CiTestCase,
-    FilesystemMockingTestCase,
-    mock,
-    skipUnlessJsonSchema,
-)
+from tests.unittests.helpers import mock, skipUnlessJsonSchema
 from tests.unittests.util import get_cloud
 
 NTP_TEMPLATE = """\
@@ -38,48 +33,37 @@ NTP={% for host in servers|list + pools|list %}{{ host }} {% endfor -%}
 """
 
 
-class TestNtp(FilesystemMockingTestCase):
-    with_logs = True
+class TestNtp:
+    @pytest.fixture
+    def service_mocks(self, mocker):
+        mocker.patch("cloudinit.config.cc_ntp.install_ntp_client")
+        mocker.patch("cloudinit.config.cc_ntp.util.is_BSD", return_value=False)
 
-    def setUp(self):
-        super(TestNtp, self).setUp()
-        self.new_root = self.tmp_dir()
-        self.add_patch("cloudinit.util.system_is_snappy", "m_snappy")
-        self.m_snappy.return_value = False
-        self.new_root = self.reRoot()
-        self._get_cloud = partial(
-            get_cloud, paths=helpers.Paths({"templates_dir": self.new_root})
-        )
-
-    def _get_template_path(self, template_name, distro, basepath=None):
+    def _get_template_path(self, template_name, distro, templates_dir):
         # ntp.conf.{distro} -> ntp.conf.debian.tmpl
         template_fn = "{0}.tmpl".format(
             template_name.replace("{distro}", distro)
         )
-        if not basepath:
-            basepath = self.new_root
-        path = os.path.join(basepath, template_fn)
+        path = os.path.join(templates_dir, template_fn)
         return path
 
-    def _generate_template(self, template=None):
-        if not template:
-            template = NTP_TEMPLATE
-        confpath = os.path.join(self.new_root, "client.conf")
-        template_fn = os.path.join(self.new_root, "client.conf.tmpl")
+    def _generate_template(self, templates_dir, template=NTP_TEMPLATE):
+        confpath = os.path.join(templates_dir, "client.conf")
+        template_fn = os.path.join(templates_dir, "client.conf.tmpl")
         util.write_file(template_fn, content=template)
         return (confpath, template_fn)
 
-    def _mock_ntp_client_config(self, client=None, distro=None):
-        if not client:
-            client = "ntp"
-        if not distro:
-            distro = "ubuntu"
+    def _mock_ntp_client_config(
+        self, templates_dir, client="ntp", distro="ubuntu"
+    ):
         dcfg = cc_ntp.distro_ntp_client_configs(distro)
         if client == "systemd-timesyncd":
             template = TIMESYNCD_TEMPLATE
         else:
             template = NTP_TEMPLATE
-        (confpath, _template_fn) = self._generate_template(template=template)
+        (confpath, _template_fn) = self._generate_template(
+            templates_dir, template=template
+        )
         ntpconfig = copy.deepcopy(dcfg[client])
         ntpconfig["confpath"] = confpath
         ntpconfig["template_name"] = os.path.basename(confpath)
@@ -117,43 +101,43 @@ class TestNtp(FilesystemMockingTestCase):
         )
         install_func.assert_called_once_with([])
 
-    def test_ntp_rename_ntp_conf(self):
+    def test_ntp_rename_ntp_conf(self, tmpdir):
         """When NTP_CONF exists, rename_ntp moves it."""
-        ntpconf = self.tmp_path("ntp.conf", self.new_root)
+        ntpconf = os.path.join(tmpdir, "ntp.conf")
         util.write_file(ntpconf, "")
         cc_ntp.rename_ntp_conf(confpath=ntpconf)
-        self.assertFalse(os.path.exists(ntpconf))
-        self.assertTrue(os.path.exists("{0}.dist".format(ntpconf)))
+        assert not os.path.exists(ntpconf)
+        assert os.path.exists("{0}.dist".format(ntpconf))
 
-    def test_ntp_rename_ntp_conf_skip_missing(self):
+    def test_ntp_rename_ntp_conf_skip_missing(self, tmp_path):
         """When NTP_CONF doesn't exist rename_ntp doesn't create a file."""
-        ntpconf = self.tmp_path("ntp.conf", self.new_root)
-        self.assertFalse(os.path.exists(ntpconf))
+        ntpconf = tmp_path / "ntp.conf"
+        assert not os.path.exists(ntpconf)
         cc_ntp.rename_ntp_conf(confpath=ntpconf)
-        self.assertFalse(os.path.exists("{0}.dist".format(ntpconf)))
-        self.assertFalse(os.path.exists(ntpconf))
+        assert not os.path.exists("{0}.dist".format(ntpconf))
+        assert not os.path.exists(ntpconf)
 
-    def test_write_ntp_config_template_uses_ntp_conf_distro_no_servers(self):
+    def test_write_ntp_config_template_uses_ntp_conf_distro_no_servers(
+        self, tmpdir
+    ):
         """write_ntp_config_template reads from $client.conf.distro.tmpl"""
-        servers = []
-        pools = ["10.0.0.1", "10.0.0.2"]
-        (confpath, template_fn) = self._generate_template()
-        mock_path = "cloudinit.config.cc_ntp.temp_utils._TMPDIR"
-        with mock.patch(mock_path, self.new_root):
-            cc_ntp.write_ntp_config_template(
-                "ubuntu",
-                servers=servers,
-                pools=pools,
-                path=confpath,
-                template_fn=template_fn,
-                template=None,
-            )
-        self.assertEqual(
-            "servers []\npools ['10.0.0.1', '10.0.0.2']\n",
-            util.load_text_file(confpath),
+        (confpath, template_fn) = self._generate_template(tmpdir)
+        cc_ntp.write_ntp_config_template(
+            "ubuntu",
+            servers=[],
+            pools=["10.0.0.1", "10.0.0.2"],
+            path=confpath,
+            template_fn=template_fn,
+            template=None,
+        )
+        assert (
+            "servers []\npools ['10.0.0.1', '10.0.0.2']\n"
+            == util.load_text_file(confpath)
         )
 
-    def test_write_ntp_config_template_defaults_pools_w_empty_lists(self):
+    def test_write_ntp_config_template_defaults_pools_w_empty_lists(
+        self, tmpdir
+    ):
         """write_ntp_config_template defaults pools servers upon empty config.
 
         When both pools and servers are empty, default NR_POOL_SERVERS get
@@ -161,24 +145,20 @@ class TestNtp(FilesystemMockingTestCase):
         """
         distro = "ubuntu"
         pools = cc_ntp.generate_server_names(distro)
-        servers = []
-        (confpath, template_fn) = self._generate_template()
-        mock_path = "cloudinit.config.cc_ntp.temp_utils._TMPDIR"
-        with mock.patch(mock_path, self.new_root):
-            cc_ntp.write_ntp_config_template(
-                distro,
-                servers=servers,
-                pools=pools,
-                path=confpath,
-                template_fn=template_fn,
-                template=None,
-            )
-        self.assertEqual(
-            "servers []\npools {0}\n".format(pools),
-            util.load_text_file(confpath),
+        (confpath, template_fn) = self._generate_template(tmpdir)
+        cc_ntp.write_ntp_config_template(
+            distro,
+            servers=[],
+            pools=pools,
+            path=confpath,
+            template_fn=template_fn,
+            template=None,
+        )
+        assert "servers []\npools {0}\n".format(pools) == util.load_text_file(
+            confpath
         )
 
-    def test_defaults_pools_empty_lists_sles(self):
+    def test_defaults_pools_empty_lists_sles(self, tmpdir, caplog):
         """write_ntp_config_template defaults opensuse pools upon empty config.
 
         When both pools and servers are empty, default NR_POOL_SERVERS get
@@ -186,7 +166,7 @@ class TestNtp(FilesystemMockingTestCase):
         """
         distro = "sles"
         default_pools = cc_ntp.generate_server_names(distro)
-        (confpath, template_fn) = self._generate_template()
+        (confpath, template_fn) = self._generate_template(tmpdir)
 
         cc_ntp.write_ntp_config_template(
             distro,
@@ -197,24 +177,23 @@ class TestNtp(FilesystemMockingTestCase):
             template=None,
         )
         for pool in default_pools:
-            self.assertIn("opensuse", pool)
-        self.assertEqual(
-            "servers []\npools {0}\n".format(default_pools),
-            util.load_text_file(confpath),
-        )
-        self.assertIn(
+            assert "opensuse" in pool
+        assert "servers []\npools {0}\n".format(
+            default_pools
+        ) == util.load_text_file(confpath)
+        assert (
             "Adding distro default ntp pool servers: {0}".format(
                 ",".join(default_pools)
-            ),
-            self.logs.getvalue(),
+            )
+            in caplog.text
         )
 
-    def test_timesyncd_template(self):
+    def test_timesyncd_template(self, tmpdir):
         """Test timesycnd template is correct"""
         pools = ["0.mycompany.pool.ntp.org", "3.mycompany.pool.ntp.org"]
         servers = ["192.168.23.3", "192.168.23.4"]
         (confpath, template_fn) = self._generate_template(
-            template=TIMESYNCD_TEMPLATE
+            tmpdir, template=TIMESYNCD_TEMPLATE
         )
         cc_ntp.write_ntp_config_template(
             "ubuntu",
@@ -224,10 +203,10 @@ class TestNtp(FilesystemMockingTestCase):
             template_fn=template_fn,
             template=None,
         )
-        self.assertEqual(
-            "[Time]\nNTP=%s %s \n" % (" ".join(servers), " ".join(pools)),
-            util.load_text_file(confpath),
-        )
+        assert "[Time]\nNTP=%s %s \n" % (
+            " ".join(servers),
+            " ".join(pools),
+        ) == util.load_text_file(confpath)
 
     def test_distro_ntp_client_configs(self):
         """Test we have updated ntp client configs on different distros"""
@@ -237,50 +216,50 @@ class TestNtp(FilesystemMockingTestCase):
         for distro in cc_ntp.distros:
             if distro not in delta:
                 result = cc_ntp.distro_ntp_client_configs(distro)
-                self.assertEqual(base, result)
+                assert base == result
         # for distros with delta, ensure the merged config values match
         # what is set in the delta
         for distro in delta.keys():
             result = cc_ntp.distro_ntp_client_configs(distro)
             for client in delta[distro].keys():
                 for key in delta[distro][client].keys():
-                    self.assertEqual(
-                        delta[distro][client][key], result[client][key]
-                    )
+                    assert delta[distro][client][key] == result[client][key]
 
-    def _get_expected_pools(self, pools, distro, client):
-        if client in ["ntp", "chrony"]:
-            if client == "ntp" and distro == "alpine":
-                # NTP for Alpine Linux is Busybox's ntp which does not
-                # support 'pool' lines in its configuration file.
-                expected_pools = []
-            else:
-                expected_pools = [
-                    "pool {0} iburst".format(pool) for pool in pools
-                ]
+    def _get_expected_pools(
+        self, pools: List[str], distro, client
+    ) -> List[str]:
+        if client == "ntp" and distro == "alpine":
+            # NTP for Alpine Linux is Busybox's ntp which does not
+            # support 'pool' lines in its configuration file.
+            expected_pools = []
+        elif client in ["ntp", "chrony"]:
+            expected_pools = ["pool {0} iburst".format(pool) for pool in pools]
         elif client == "systemd-timesyncd":
-            expected_pools = " ".join(pools)
+            expected_pools = pools
+        else:
+            raise RuntimeError(f"Unknown client: {client}")
 
         return expected_pools
 
-    def _get_expected_servers(self, servers, distro, client):
-        if client in ["ntp", "chrony"]:
-            if client == "ntp" and distro == "alpine":
-                # NTP for Alpine Linux is Busybox's ntp which only supports
-                # 'server' lines without iburst option.
-                expected_servers = [
-                    "server {0}".format(srv) for srv in servers
-                ]
-            else:
-                expected_servers = [
-                    "server {0} iburst".format(srv) for srv in servers
-                ]
+    def _get_expected_servers(
+        self, servers: List[str], distro, client
+    ) -> List[str]:
+        if client == "ntp" and distro == "alpine":
+            # NTP for Alpine Linux is Busybox's ntp which only supports
+            # 'server' lines without iburst option.
+            expected_servers = ["server {0}".format(srv) for srv in servers]
+        elif client in ["ntp", "chrony"]:
+            expected_servers = [
+                "server {0} iburst".format(srv) for srv in servers
+            ]
         elif client == "systemd-timesyncd":
-            expected_servers = " ".join(servers)
+            expected_servers = servers
+        else:
+            raise RuntimeError(f"Unknown client: {client}")
 
         return expected_servers
 
-    def test_ntp_handler_real_distro_ntp_templates(self):
+    def test_ntp_handler_real_distro_ntp_templates(self, tmpdir):
         """Test ntp handler renders the shipped distro ntp client templates."""
         pools = ["0.mycompany.pool.ntp.org", "3.mycompany.pool.ntp.org"]
         servers = ["192.168.23.3", "192.168.23.4"]
@@ -288,9 +267,7 @@ class TestNtp(FilesystemMockingTestCase):
             for distro in cc_ntp.distros:
                 distro_cfg = cc_ntp.distro_ntp_client_configs(distro)
                 ntpclient = distro_cfg[client]
-                confpath = os.path.join(
-                    self.new_root, ntpclient.get("confpath")[1:]
-                )
+                confpath = os.path.join(tmpdir, ntpclient["confpath"][1:])
                 template = ntpclient.get("template_name")
                 # find sourcetree template file
                 root_dir = (
@@ -298,9 +275,9 @@ class TestNtp(FilesystemMockingTestCase):
                     + "/templates"
                 )
                 source_fn = self._get_template_path(
-                    template, distro, basepath=root_dir
+                    template, distro, templates_dir=root_dir
                 )
-                template_fn = self._get_template_path(template, distro)
+                template_fn = self._get_template_path(template, distro, tmpdir)
                 # don't fail if cloud-init doesn't have a template for
                 # a distro,client pair
                 if not os.path.exists(source_fn):
@@ -320,25 +297,21 @@ class TestNtp(FilesystemMockingTestCase):
                     expected_servers = self._get_expected_servers(
                         servers, distro, client
                     )
-                    print("distro=%s client=%s" % (distro, client))
+                    print(f"distro={distro} client={client}")
                     for sline in expected_servers:
-                        self.assertIn(
-                            sline,
-                            content_lines,
-                            "failed to render {0} conf for distro:{1}".format(
-                                client, distro
-                            ),
+                        assert (
+                            sline in content_lines
+                        ), "failed to render {0} conf for distro:{1}".format(
+                            client, distro
                         )
                     expected_pools = self._get_expected_pools(
                         pools, distro, client
                     )
                     if expected_pools != []:
                         for pline in expected_pools:
-                            self.assertIn(
-                                pline,
-                                content_lines,
+                            assert pline in content_lines, (
                                 "failed to render {0} conf"
-                                " for distro:{1}".format(client, distro),
+                                " for distro:{1}".format(client, distro)
                             )
                 elif client == "systemd-timesyncd":
                     expected_servers = self._get_expected_servers(
@@ -351,74 +324,79 @@ class TestNtp(FilesystemMockingTestCase):
                         "# cloud-init generated file\n"
                         "# See timesyncd.conf(5) for details.\n\n"
                         "[Time]\nNTP=%s %s \n"
-                        % (expected_servers, expected_pools)
+                        % (
+                            " ".join(expected_servers),
+                            " ".join(expected_pools),
+                        )
                     )
-                    self.assertEqual(expected_content, content)
+                    assert expected_content == content
 
-    def test_no_ntpcfg_does_nothing(self):
+    def test_no_ntpcfg_does_nothing(self, caplog):
         """When no ntp section is defined handler logs a warning and noops."""
-        cc_ntp.handle("cc_ntp", {}, None, [])
-        self.assertEqual(
-            "DEBUG: Skipping module named cc_ntp, "
-            "not present or disabled by cfg\n",
-            self.logs.getvalue(),
+        cc_ntp.handle("cc_ntp", {}, get_cloud(), [])
+        assert (
+            "Skipping module named cc_ntp, "
+            "not present or disabled by cfg\n" in caplog.text
         )
 
-    @mock.patch("cloudinit.config.cc_ntp.select_ntp_client")
     def test_ntp_handler_schema_validation_allows_empty_ntp_config(
-        self, m_select
+        self, service_mocks, mocker, paths, caplog
     ):
         """Ntp schema validation allows for an empty ntp: configuration."""
-        valid_empty_configs = [{"ntp": {}}, {"ntp": None}]
+        m_select = mocker.patch("cloudinit.config.cc_ntp.select_ntp_client")
+
+        valid_empty_configs: List[Dict] = [{"ntp": {}}, {"ntp": None}]
         for valid_empty_config in valid_empty_configs:
             for distro in cc_ntp.distros:
                 # skip the test if the distro is COS. As in COS, the default
                 # config file is installed
                 if distro == "cos":
-                    return
-                mycloud = self._get_cloud(distro)
-                ntpconfig = self._mock_ntp_client_config(distro=distro)
+                    continue
+                mycloud = get_cloud(distro, paths=paths)
+                ntpconfig = self._mock_ntp_client_config(
+                    paths.cfgs["templates_dir"], distro=distro
+                )
                 confpath = ntpconfig["confpath"]
                 m_select.return_value = ntpconfig
-                cc_ntp.handle("cc_ntp", valid_empty_config, mycloud, [])
+
+                with mock.patch.object(mycloud.distro, "manage_service"):
+                    cc_ntp.handle("cc_ntp", valid_empty_config, mycloud, [])
+
                 if distro == "alpine":
                     # _mock_ntp_client_config call above did not specify a
                     # client value and so it defaults to "ntp" which on
                     # Alpine Linux only supports servers and not pools.
 
                     servers = cc_ntp.generate_server_names(mycloud.distro.name)
-                    self.assertEqual(
-                        "servers {0}\npools []\n".format(servers),
-                        util.load_text_file(confpath),
-                    )
+                    assert "servers {0}\npools []\n".format(
+                        servers
+                    ) == util.load_text_file(confpath)
                 else:
                     pools = cc_ntp.generate_server_names(mycloud.distro.name)
-                    self.assertEqual(
-                        "servers []\npools {0}\n".format(pools),
-                        util.load_text_file(confpath),
-                    )
-            self.assertNotIn(
-                "Invalid cloud-config provided:", self.logs.getvalue()
-            )
+                    assert "servers []\npools {0}\n".format(
+                        pools
+                    ) == util.load_text_file(confpath)
+            assert "Invalid cloud-config provided:" not in caplog.text
 
-    @mock.patch("cloudinit.config.cc_ntp.select_ntp_client")
-    def test_ntp_handler_timesyncd(self, m_select):
+    def test_ntp_handler_timesyncd(self, service_mocks, mocker, tmpdir, paths):
         """Test ntp handler configures timesyncd"""
+        m_select = mocker.patch("cloudinit.config.cc_ntp.select_ntp_client")
         servers = ["192.168.2.1", "192.168.2.2"]
         pools = ["0.mypool.org"]
         cfg = {"ntp": {"servers": servers, "pools": pools}}
         client = "systemd-timesyncd"
         for distro in cc_ntp.distros:
-            mycloud = self._get_cloud(distro)
+            mycloud = get_cloud(distro, paths=paths)
             ntpconfig = self._mock_ntp_client_config(
-                distro=distro, client=client
+                paths.cfgs["templates_dir"], distro=distro, client=client
             )
             confpath = ntpconfig["confpath"]
             m_select.return_value = ntpconfig
-            cc_ntp.handle("cc_ntp", cfg, mycloud, [])
-            self.assertEqual(
-                "[Time]\nNTP=192.168.2.1 192.168.2.2 0.mypool.org \n",
-                util.load_text_file(confpath),
+            with mock.patch.object(mycloud.distro, "manage_service"):
+                cc_ntp.handle("cc_ntp", cfg, mycloud, [])
+            assert (
+                "[Time]\nNTP=192.168.2.1 192.168.2.2 0.mypool.org \n"
+                == util.load_text_file(confpath)
             )
 
     @mock.patch("cloudinit.config.cc_ntp.select_ntp_client")
@@ -426,21 +404,25 @@ class TestNtp(FilesystemMockingTestCase):
         """Test ntp handler does not run if enabled: false"""
         cfg = {"ntp": {"enabled": False}}
         for distro in cc_ntp.distros:
-            mycloud = self._get_cloud(distro)
-            cc_ntp.handle("notimportant", cfg, mycloud, None)
-            self.assertEqual(0, m_select.call_count)
+            mycloud = get_cloud(distro)
+            cc_ntp.handle("notimportant", cfg, mycloud, [])
+            assert 0 == m_select.call_count
 
     @mock.patch("cloudinit.subp.subp")
     @mock.patch("cloudinit.subp.which", return_value=True)
     @mock.patch("cloudinit.config.cc_ntp.select_ntp_client")
     @mock.patch("cloudinit.distros.Distro.uses_systemd")
-    def test_ntp_the_whole_package(self, m_sysd, m_select, m_which, m_subp):
+    def test_ntp_the_whole_package(
+        self, m_sysd, m_select, m_which, m_subp, tmpdir, paths
+    ):
         """Test enabled config renders template, and restarts service"""
         cfg = {"ntp": {"enabled": True}}
         for distro in cc_ntp.distros:
             m_subp.reset_mock()
-            mycloud = self._get_cloud(distro)
-            ntpconfig = self._mock_ntp_client_config(distro=distro)
+            mycloud = get_cloud(distro, paths=paths)
+            ntpconfig = self._mock_ntp_client_config(
+                paths.cfgs["templates_dir"], distro=distro
+            )
             confpath = ntpconfig["confpath"]
             service_name = ntpconfig["service_name"]
             m_select.return_value = ntpconfig
@@ -459,7 +441,7 @@ class TestNtp(FilesystemMockingTestCase):
             # skip the test if the distro is COS. As in COS, the default
             # config file is installed
             if distro == "cos":
-                return
+                continue
 
             if distro == "alpine":
                 uses_systemd = False
@@ -502,45 +484,41 @@ class TestNtp(FilesystemMockingTestCase):
                 m_util.is_BSD.return_value = is_FreeBSD or is_OpenBSD
                 m_util.is_FreeBSD.return_value = is_FreeBSD
                 m_util.is_OpenBSD.return_value = is_OpenBSD
-                cc_ntp.handle("notimportant", cfg, mycloud, None)
+                cc_ntp.handle("notimportant", cfg, mycloud, [])
                 m_subp.assert_called_with(
                     expected_service_call, capture=True, rcs=None
                 )
 
-            self.assertEqual(expected_content, util.load_text_file(confpath))
+            assert expected_content == util.load_text_file(confpath)
 
     @mock.patch("cloudinit.util.system_info")
     def test_opensuse_picks_chrony(self, m_sysinfo):
         """Test opensuse picks chrony or ntp on certain distro versions"""
         #  < 15.0  => ntp
         m_sysinfo.return_value = {"dist": ("openSUSE", "13.2", "Harlequin")}
-        mycloud = self._get_cloud("opensuse")
+        mycloud = get_cloud("opensuse")
         expected_client = mycloud.distro.preferred_ntp_clients[0]
-        self.assertEqual("ntp", expected_client)
+        assert "ntp" == expected_client
 
         #  >= 15.0 and  not openSUSE => chrony
         m_sysinfo.return_value = {
             "dist": ("SLES", "15.0", "SUSE Linux Enterprise Server 15")
         }
-        mycloud = self._get_cloud("sles")
+        mycloud = get_cloud("sles")
         expected_client = mycloud.distro.preferred_ntp_clients[0]
-        self.assertEqual("chrony", expected_client)
+        assert "chrony" == expected_client
 
         #  >= 15.0 and  openSUSE and ver != 42  => chrony
         m_sysinfo.return_value = {
             "dist": ("openSUSE Tumbleweed", "20180326", "timbleweed")
         }
-        mycloud = self._get_cloud("opensuse")
+        mycloud = get_cloud("opensuse")
         expected_client = mycloud.distro.preferred_ntp_clients[0]
-        self.assertEqual("chrony", expected_client)
+        assert "chrony" == expected_client
 
     @mock.patch("cloudinit.config.cc_ntp.subp.which")
     def test_snappy_system_picks_timesyncd(self, m_which):
         """Test snappy systems prefer installed clients"""
-
-        # we are on ubuntu-core here
-        self.m_snappy.return_value = True
-
         # ubuntu core systems will have timesyncd installed, so simulate that.
         # First None is for the 'eatmydata' check when initializing apt
         # when initializing the distro class. The rest represent possible
@@ -549,19 +527,19 @@ class TestNtp(FilesystemMockingTestCase):
             [None, None, "/lib/systemd/systemd-timesyncd", None, None, None]
         )
         distro = "ubuntu"
-        mycloud = self._get_cloud(distro)
+        mycloud = get_cloud(distro)
         distro_configs = cc_ntp.distro_ntp_client_configs(distro)
         expected_client = "systemd-timesyncd"
         expected_cfg = distro_configs[expected_client]
         expected_calls = []
         # we only get to timesyncd
-        for client in mycloud.distro.preferred_ntp_clients[0:2]:
+        for client in mycloud.distro.preferred_ntp_clients[:2]:
             cfg = distro_configs[client]
             expected_calls.append(mock.call(cfg["check_exe"]))
         result = cc_ntp.select_ntp_client(None, mycloud.distro)
         m_which.assert_has_calls(expected_calls)
-        self.assertEqual(sorted(expected_cfg), sorted(cfg))
-        self.assertEqual(sorted(expected_cfg), sorted(result))
+        assert sorted(expected_cfg) == sorted(cfg)
+        assert sorted(expected_cfg) == sorted(result)
 
     @mock.patch("cloudinit.config.cc_ntp.subp.which")
     def test_ntp_distro_searches_all_preferred_clients(self, m_which):
@@ -569,7 +547,7 @@ class TestNtp(FilesystemMockingTestCase):
         # nothing is installed
         m_which.return_value = None
         for distro in cc_ntp.distros:
-            mycloud = self._get_cloud(distro)
+            mycloud = get_cloud(distro)
             distro_configs = cc_ntp.distro_ntp_client_configs(distro)
             expected_client = mycloud.distro.preferred_ntp_clients[0]
             expected_cfg = distro_configs[expected_client]
@@ -579,7 +557,7 @@ class TestNtp(FilesystemMockingTestCase):
                 expected_calls.append(mock.call(cfg["check_exe"]))
             cc_ntp.select_ntp_client({}, mycloud.distro)
             m_which.assert_has_calls(expected_calls)
-            self.assertEqual(sorted(expected_cfg), sorted(cfg))
+            assert sorted(expected_cfg) == sorted(cfg)
 
     @mock.patch("cloudinit.config.cc_ntp.subp.which")
     def test_user_cfg_ntp_client_auto_uses_distro_clients(self, m_which):
@@ -587,7 +565,7 @@ class TestNtp(FilesystemMockingTestCase):
         # nothing is installed
         m_which.return_value = None
         for distro in cc_ntp.distros:
-            mycloud = self._get_cloud(distro)
+            mycloud = get_cloud(distro)
             distro_configs = cc_ntp.distro_ntp_client_configs(distro)
             expected_client = mycloud.distro.preferred_ntp_clients[0]
             expected_cfg = distro_configs[expected_client]
@@ -597,7 +575,7 @@ class TestNtp(FilesystemMockingTestCase):
                 expected_calls.append(mock.call(cfg["check_exe"]))
             cc_ntp.select_ntp_client("auto", mycloud.distro)
             m_which.assert_has_calls(expected_calls)
-            self.assertEqual(sorted(expected_cfg), sorted(cfg))
+            assert sorted(expected_cfg) == sorted(cfg)
 
     @mock.patch("cloudinit.config.cc_ntp.write_ntp_config_template")
     @mock.patch("cloudinit.cloud.Cloud.get_template_filename")
@@ -611,11 +589,13 @@ class TestNtp(FilesystemMockingTestCase):
         for distro in cc_ntp.distros:
             # client is not installed
             m_which.return_value = None
-            mycloud = self._get_cloud(distro)
+            mycloud = get_cloud(distro)
             with mock.patch.object(
                 mycloud.distro, "install_packages"
-            ) as m_install:
-                cc_ntp.handle("notimportant", cfg, mycloud, None)
+            ) as m_install, mock.patch.object(
+                mycloud.distro, "manage_service"
+            ):
+                cc_ntp.handle("notimportant", cfg, mycloud, [])
             m_install.assert_called_with([client])
             m_which.assert_called_with(client)
 
@@ -627,11 +607,11 @@ class TestNtp(FilesystemMockingTestCase):
         # no clients installed
         m_which.return_value = None
         for distro in cc_ntp.distros:
-            mycloud = self._get_cloud(distro, sys_cfg=sys_cfg)
+            mycloud = get_cloud(distro, sys_cfg=sys_cfg)
             distro_configs = cc_ntp.distro_ntp_client_configs(distro)
             expected_cfg = distro_configs[system_client]
             result = cc_ntp.select_ntp_client(None, mycloud.distro)
-            self.assertEqual(sorted(expected_cfg), sorted(result))
+            assert sorted(expected_cfg) == sorted(result)
             m_which.assert_has_calls([])
 
     @mock.patch("cloudinit.config.cc_ntp.subp.which")
@@ -643,18 +623,18 @@ class TestNtp(FilesystemMockingTestCase):
         # no clients installed
         m_which.return_value = None
         for distro in cc_ntp.distros:
-            mycloud = self._get_cloud(distro, sys_cfg=sys_cfg)
+            mycloud = get_cloud(distro, sys_cfg=sys_cfg)
             distro_configs = cc_ntp.distro_ntp_client_configs(distro)
             expected_cfg = distro_configs[user_client]
             result = cc_ntp.select_ntp_client(user_client, mycloud.distro)
-            self.assertEqual(sorted(expected_cfg), sorted(result))
+            assert sorted(expected_cfg) == sorted(result)
             m_which.assert_has_calls([])
 
     @mock.patch("cloudinit.config.cc_ntp.install_ntp_client")
-    def test_ntp_user_provided_config_with_template(self, m_install):
+    def test_ntp_user_provided_config_with_template(self, m_install, tmpdir):
         custom = r"\n#MyCustomTemplate"
         user_template = NTP_TEMPLATE + custom
-        confpath = os.path.join(self.new_root, "etc/myntp/myntp.conf")
+        confpath = os.path.join(tmpdir, "etc/myntp/myntp.conf")
         cfg = {
             "ntp": {
                 "pools": ["mypool.org"],
@@ -668,21 +648,23 @@ class TestNtp(FilesystemMockingTestCase):
                 },
             }
         }
+        mock_path = "cloudinit.config.cc_ntp.temp_utils.get_tmp_ancestor"
         for distro in cc_ntp.distros:
-            mycloud = self._get_cloud(distro)
-            mock_path = "cloudinit.config.cc_ntp.temp_utils._TMPDIR"
-            with mock.patch(mock_path, self.new_root):
-                cc_ntp.handle("notimportant", cfg, mycloud, None)
-            self.assertEqual(
-                "servers []\npools ['mypool.org']\n%s" % custom,
-                util.load_text_file(confpath),
+            mycloud = get_cloud(distro)
+            with mock.patch(mock_path, lambda *_: tmpdir), mock.patch.object(
+                mycloud.distro, "manage_service"
+            ):
+                cc_ntp.handle("notimportant", cfg, mycloud, [])
+            assert (
+                "servers []\npools ['mypool.org']\n%s" % custom
+                == util.load_text_file(confpath)
             )
 
     @mock.patch("cloudinit.config.cc_ntp.supplemental_schema_validation")
     @mock.patch("cloudinit.config.cc_ntp.install_ntp_client")
     @mock.patch("cloudinit.config.cc_ntp.select_ntp_client")
     def test_ntp_user_provided_config_template_only(
-        self, m_select, m_install, m_schema
+        self, m_select, m_install, m_schema, tmpdir
     ):
         """Test custom template for default client"""
         custom = r"\n#MyCustomTemplate"
@@ -697,43 +679,44 @@ class TestNtp(FilesystemMockingTestCase):
         }
         expected_merged_cfg = {
             "check_exe": "chronyd",
-            "confpath": "{tmpdir}/client.conf".format(tmpdir=self.new_root),
+            "confpath": f"{tmpdir}/client.conf",
             "template_name": "client.conf",
             "template": user_template,
             "service_name": "chrony",
             "packages": ["chrony"],
         }
         for distro in cc_ntp.distros:
-            mycloud = self._get_cloud(distro)
+            mycloud = get_cloud(distro)
             ntpconfig = self._mock_ntp_client_config(
-                client=client, distro=distro
+                tmpdir, client=client, distro=distro
             )
             confpath = ntpconfig["confpath"]
             m_select.return_value = ntpconfig
-            mock_path = "cloudinit.config.cc_ntp.temp_utils._TMPDIR"
-            with mock.patch(mock_path, self.new_root):
-                cc_ntp.handle("notimportant", {"ntp": cfg}, mycloud, None)
-            self.assertEqual(
-                "servers []\npools ['mypool.org']\n%s" % custom,
-                util.load_text_file(confpath),
+            mock_path = "cloudinit.config.cc_ntp.temp_utils.get_tmp_ancestor"
+            with mock.patch(mock_path, lambda *_: tmpdir), mock.patch.object(
+                mycloud.distro, "manage_service"
+            ):
+                cc_ntp.handle("notimportant", {"ntp": cfg}, mycloud, [])
+            assert (
+                "servers []\npools ['mypool.org']\n%s" % custom
+                == util.load_text_file(confpath)
             )
         m_schema.assert_called_with(expected_merged_cfg)
 
 
-class TestSupplementalSchemaValidation(CiTestCase):
+class TestSupplementalSchemaValidation:
     def test_error_on_missing_keys(self):
         """ValueError raised reporting any missing required ntp:config keys"""
-        cfg = {}
         match = (
             r"Invalid ntp configuration:\\nMissing required ntp:config"
             " keys: check_exe, confpath, packages, service_name"
         )
-        with self.assertRaisesRegex(ValueError, match):
-            cc_ntp.supplemental_schema_validation(cfg)
+        with pytest.raises(ValueError, match=match):
+            cc_ntp.supplemental_schema_validation({})
 
     def test_error_requiring_either_template_or_template_name(self):
         """ValueError raised if both template not template_name are None."""
-        cfg = {
+        cfg: Dict[str, Any] = {
             "confpath": "someconf",
             "check_exe": "",
             "service_name": "",
@@ -745,7 +728,7 @@ class TestSupplementalSchemaValidation(CiTestCase):
             r"Invalid ntp configuration:\\nEither ntp:config:template"
             " or ntp:config:template_name values are required"
         )
-        with self.assertRaisesRegex(ValueError, match):
+        with pytest.raises(ValueError, match=match):
             cc_ntp.supplemental_schema_validation(cfg)
 
     def test_error_on_non_list_values(self):
@@ -762,7 +745,7 @@ class TestSupplementalSchemaValidation(CiTestCase):
             r"Invalid ntp configuration:\\nExpected a list of required"
             " package names for ntp:config:packages. Found \\(NOPE\\)"
         )
-        with self.assertRaisesRegex(ValueError, match):
+        with pytest.raises(ValueError, match=match):
             cc_ntp.supplemental_schema_validation(cfg)
 
     def test_error_on_non_string_values(self):
@@ -782,11 +765,11 @@ class TestSupplementalSchemaValidation(CiTestCase):
             "Expected a string type for ntp:config:template. Found (4)",
             "Expected a string type for ntp:config:template_name. Found (5)",
         ]
-        with self.assertRaises(ValueError) as context_mgr:
+        with pytest.raises(ValueError) as context_mgr:
             cc_ntp.supplemental_schema_validation(cfg)
-        error_msg = str(context_mgr.exception)
+        error_msg = str(context_mgr.value)
         for error in errors:
-            self.assertIn(error, error_msg)
+            assert error in error_msg
 
 
 class TestNTPSchema:
