@@ -22,7 +22,7 @@ from cloudinit.config.schema import (
     get_schema,
     validate_cloudconfig_schema,
 )
-from cloudinit.subp import ProcessExecutionError
+from cloudinit.subp import ProcessExecutionError, SubpResult
 from tests.unittests.helpers import (
     CiTestCase,
     mock,
@@ -62,7 +62,7 @@ class TestResizefs(CiTestCase):
         fs_type = "ufs"
         resize_what = "/"
         devpth = "/dev/da0p2"
-        m_subp.return_value = (
+        m_subp.return_value = SubpResult(
             "stdout: super-block backups (for fsck_ffs -b #) at:\n\n",
             "growfs: no room to allocate last cylinder group; "
             "leaving 364KB unused\n",
@@ -457,7 +457,7 @@ class TestMaybeGetDevicePathAsWritableBlock(CiTestCase):
         """Do not resize / directly if it is read-only. (LP: #1734787)."""
         m_is_rw.return_value = False
         m_is_dir.return_value = True
-        m_subp.return_value = ("btrfs-progs v4.19 \n", "")
+        m_subp.return_value = SubpResult("btrfs-progs v4.19 \n", "")
         self.assertEqual(
             ("btrfs", "filesystem", "resize", "max", "//.snapshots"),
             _resize_btrfs("/", "/dev/sda1"),
@@ -470,7 +470,7 @@ class TestMaybeGetDevicePathAsWritableBlock(CiTestCase):
         """Do not resize / directly if it is read-only. (LP: #1734787)."""
         m_is_rw.return_value = True
         m_is_dir.return_value = True
-        m_subp.return_value = ("btrfs-progs v4.19 \n", "")
+        m_subp.return_value = SubpResult("btrfs-progs v4.19 \n", "")
         self.assertEqual(
             ("btrfs", "filesystem", "resize", "max", "/"),
             _resize_btrfs("/", "/dev/sda1"),
@@ -485,7 +485,24 @@ class TestMaybeGetDevicePathAsWritableBlock(CiTestCase):
         """Queue the resize request if btrfs >= 5.10"""
         m_is_rw.return_value = True
         m_is_dir.return_value = True
-        m_subp.return_value = ("btrfs-progs v5.10 \n", "")
+        m_subp.return_value = SubpResult("btrfs-progs v5.10 \n", "")
+        self.assertEqual(
+            ("btrfs", "filesystem", "resize", "--enqueue", "max", "/"),
+            _resize_btrfs("/", "/dev/sda1"),
+        )
+
+    @mock.patch("cloudinit.util.mount_is_read_write")
+    @mock.patch("cloudinit.config.cc_resizefs.os.path.isdir")
+    @mock.patch("cloudinit.subp.subp")
+    def test_resize_btrfs_version(self, m_subp, m_is_dir, m_is_rw):
+        """Queue the resize request if btrfs >= 6.10"""
+        m_is_rw.return_value = True
+        m_is_dir.return_value = True
+        m_subp.return_value = SubpResult(
+            "btrfs-progs v6.10 \n\n-EXPERIMENTAL -INJECT -STATIC +LZO +ZSTD "
+            "+UDEV +FSVERITY +ZONED CRYPTO=libgcrypt",
+            "",
+        )
         self.assertEqual(
             ("btrfs", "filesystem", "resize", "--enqueue", "max", "/"),
             _resize_btrfs("/", "/dev/sda1"),
@@ -555,12 +572,12 @@ class TestZpool:
 
     @mock.patch(M_PATH + "os")
     @mock.patch("cloudinit.subp.subp")
-    def test_get_device_info_from_zpool_on_error(self, zpool_output, m_os):
+    def test_get_device_info_from_zpool_on_error(self, m_subp, m_os):
         # mock /dev/zfs exists
         m_os.path.exists.return_value = True
 
         # mock subp command from get_mount_info_fs_on_zpool
-        zpool_output.return_value = (
+        m_subp.return_value = SubpResult(
             readResource("zpool_status_simple.txt"),
             "error",
         )
