@@ -1,4 +1,5 @@
 # This file is part of cloud-init. See LICENSE file for license information.
+# pylint: disable=attribute-defined-outside-init
 
 import copy
 import errno
@@ -9,19 +10,13 @@ from typing import Optional
 from unittest import mock
 
 import pytest
-import requests
-import responses
 
 import cloudinit.net as net
 from cloudinit import subp
 from cloudinit.net.ephemeral import EphemeralIPv4Network, EphemeralIPv6Network
 from cloudinit.subp import ProcessExecutionError
 from cloudinit.util import ensure_file, write_file
-from tests.unittests.helpers import (
-    CiTestCase,
-    ResponsesTestCase,
-    random_string,
-)
+from tests.unittests.helpers import CiTestCase, example_netdev, random_string
 from tests.unittests.util import MockDistro
 
 
@@ -42,10 +37,7 @@ class TestSysDevPath:
 
 class TestReadSysNet:
     @pytest.fixture(autouse=True)
-    @pytest.mark.parametrize(
-        "disable_sysfs_net", [False], indirect=["disable_sysfs_net"]
-    )
-    def setup(self, disable_sysfs_net, tmpdir_factory):
+    def setup(self, tmpdir_factory):
         # We mock invididual numbered tmpdirs here because these tests write
         # to the sysfs directory and stale test artifacts break later tests.
         mock_sysfs = f"{tmpdir_factory.mktemp('sysfs', numbered=True)}/"
@@ -767,6 +759,7 @@ class TestInterfaceHasOwnMAC(CiTestCase):
 
 
 @mock.patch("cloudinit.net.subp.subp")
+@pytest.mark.usefixtures("disable_netdev_info")
 class TestEphemeralIPV4Network(CiTestCase):
 
     with_logs = True
@@ -791,7 +784,11 @@ class TestEphemeralIPV4Network(CiTestCase):
             params = copy.deepcopy(required_params)
             params[key] = None
             with self.assertRaises(ValueError) as context_manager:
-                EphemeralIPv4Network(MockDistro(), **params)
+                EphemeralIPv4Network(
+                    MockDistro(),
+                    interface_addrs_before_dhcp=example_netdev,
+                    **params,
+                )
             error = context_manager.exception
             self.assertIn("Cannot init network on", str(error))
             self.assertEqual(0, m_subp.call_count)
@@ -802,6 +799,7 @@ class TestEphemeralIPV4Network(CiTestCase):
             "interface": "eth0",
             "ip": "192.168.2.2",
             "broadcast": "192.168.2.255",
+            "interface_addrs_before_dhcp": example_netdev,
         }
         invalid_masks = ("invalid", "invalid.", "123.123.123")
         for error_val in invalid_masks:
@@ -831,25 +829,9 @@ class TestEphemeralIPV4Network(CiTestCase):
                     "dev",
                     "eth0",
                 ],
-                update_env={"LANG": "C"},
-            ),
-            mock.call(
-                ["ip", "-family", "inet", "link", "set", "dev", "eth0", "up"],
             ),
         ]
         expected_teardown_calls = [
-            mock.call(
-                [
-                    "ip",
-                    "-family",
-                    "inet",
-                    "link",
-                    "set",
-                    "dev",
-                    "eth0",
-                    "down",
-                ],
-            ),
             mock.call(
                 [
                     "ip",
@@ -868,6 +850,7 @@ class TestEphemeralIPV4Network(CiTestCase):
             "ip": "192.168.2.2",
             "prefix_or_mask": "255.255.255.0",
             "broadcast": "192.168.2.255",
+            "interface_addrs_before_dhcp": example_netdev,
         }
         with EphemeralIPv4Network(MockDistro(), **params):
             self.assertEqual(expected_setup_calls, m_subp.call_args_list)
@@ -894,6 +877,7 @@ class TestEphemeralIPV4Network(CiTestCase):
                 ip="1.1.1.1",
                 prefix_or_mask="255.255.255.0",
                 broadcast="1.1.1.255",
+                interface_addrs_before_dhcp=example_netdev,
                 static_routes=[
                     ("2.2.2.2/32", "9.9.9.9"),
                     ("3.3.3.3/32", "8.8.8.8"),
@@ -915,18 +899,6 @@ class TestEphemeralIPV4Network(CiTestCase):
                     "eth0",
                 ],
             ),
-            mock.call(
-                [
-                    "ip",
-                    "-family",
-                    "inet",
-                    "link",
-                    "set",
-                    "dev",
-                    "eth0",
-                    "down",
-                ],
-            ),
             mock.call(
                 [
                     "ip",
@@ -943,28 +915,6 @@ class TestEphemeralIPV4Network(CiTestCase):
         for teardown in expected_teardown_calls:
             assert teardown in m_subp.call_args_list
 
-    @mock.patch("cloudinit.net.readurl")
-    def test_ephemeral_ipv4_no_network_if_url_connectivity(
-        self, m_readurl, m_subp
-    ):
-        """No network setup is performed if we can successfully connect to
-        connectivity_url."""
-        params = {
-            "interface": "eth0",
-            "ip": "192.168.2.2",
-            "prefix_or_mask": "255.255.255.0",
-            "broadcast": "192.168.2.255",
-            "connectivity_url_data": {"url": "http://example.org/index.html"},
-        }
-
-        with EphemeralIPv4Network(MockDistro(), **params):
-            self.assertEqual(
-                [mock.call(url="http://example.org/index.html", timeout=5)],
-                m_readurl.call_args_list,
-            )
-        # Ensure that no teardown happens:
-        m_subp.assert_has_calls([])
-
     def test_ephemeral_ipv4_network_noop_when_configured(self, m_subp):
         """EphemeralIPv4Network handles exception when address is setup.
 
@@ -972,37 +922,20 @@ class TestEphemeralIPV4Network(CiTestCase):
         """
         params = {
             "interface": "eth0",
-            "ip": "192.168.2.2",
+            "ip": "10.85.130.116",
             "prefix_or_mask": "255.255.255.0",
             "broadcast": "192.168.2.255",
+            "interface_addrs_before_dhcp": example_netdev,
         }
         m_subp.side_effect = ProcessExecutionError(
             "", "RTNETLINK answers: File exists", 2
         )
-        expected_calls = [
-            mock.call(
-                [
-                    "ip",
-                    "-family",
-                    "inet",
-                    "addr",
-                    "add",
-                    "192.168.2.2/24",
-                    "broadcast",
-                    "192.168.2.255",
-                    "dev",
-                    "eth0",
-                ],
-                update_env={"LANG": "C"},
-            )
-        ]
+        expected_calls = []
         with EphemeralIPv4Network(MockDistro(), **params):
             pass
-        self.assertEqual(expected_calls, m_subp.call_args_list)
-        self.assertIn(
-            "Skip ephemeral network setup, eth0 already has address",
-            self.logs.getvalue(),
-        )
+        assert expected_calls == m_subp.call_args_list
+        assert "Skip bringing up network link" in self.logs.getvalue()
+        assert "Skip adding ip address" in self.logs.getvalue()
 
     def test_ephemeral_ipv4_network_with_prefix(self, m_subp):
         """EphemeralIPv4Network takes a valid prefix to setup the network."""
@@ -1011,6 +944,7 @@ class TestEphemeralIPV4Network(CiTestCase):
             "ip": "192.168.2.2",
             "prefix_or_mask": "24",
             "broadcast": "192.168.2.255",
+            "interface_addrs_before_dhcp": example_netdev,
         }
         for prefix_val in ["24", 16]:  # prefix can be int or string
             params["prefix_or_mask"] = prefix_val
@@ -1031,7 +965,6 @@ class TestEphemeralIPV4Network(CiTestCase):
                         "dev",
                         "eth0",
                     ],
-                    update_env={"LANG": "C"},
                 )
             ]
         )
@@ -1050,7 +983,6 @@ class TestEphemeralIPV4Network(CiTestCase):
                         "dev",
                         "eth0",
                     ],
-                    update_env={"LANG": "C"},
                 )
             ]
         )
@@ -1063,6 +995,7 @@ class TestEphemeralIPV4Network(CiTestCase):
             "prefix_or_mask": "255.255.255.0",
             "broadcast": "192.168.2.255",
             "router": "192.168.2.1",
+            "interface_addrs_before_dhcp": example_netdev,
         }
         # Empty response from ip route gw check
         m_subp.return_value = subp.SubpResult("", "")
@@ -1080,10 +1013,6 @@ class TestEphemeralIPV4Network(CiTestCase):
                     "dev",
                     "eth0",
                 ],
-                update_env={"LANG": "C"},
-            ),
-            mock.call(
-                ["ip", "-family", "inet", "link", "set", "dev", "eth0", "up"],
             ),
             mock.call(["ip", "route", "show", "0.0.0.0/0"]),
             mock.call(
@@ -1091,7 +1020,7 @@ class TestEphemeralIPV4Network(CiTestCase):
                     "ip",
                     "-4",
                     "route",
-                    "add",
+                    "replace",
                     "192.168.2.1",
                     "dev",
                     "eth0",
@@ -1104,7 +1033,7 @@ class TestEphemeralIPV4Network(CiTestCase):
                     "ip",
                     "-4",
                     "route",
-                    "add",
+                    "replace",
                     "default",
                     "via",
                     "192.168.2.1",
@@ -1148,6 +1077,7 @@ class TestEphemeralIPV4Network(CiTestCase):
                 ("0.0.0.0/0", "192.168.2.1"),
             ],
             "router": "192.168.2.1",
+            "interface_addrs_before_dhcp": example_netdev,
         }
         expected_setup_calls = [
             mock.call(
@@ -1163,10 +1093,6 @@ class TestEphemeralIPV4Network(CiTestCase):
                     "dev",
                     "eth0",
                 ],
-                update_env={"LANG": "C"},
-            ),
-            mock.call(
-                ["ip", "-family", "inet", "link", "set", "dev", "eth0", "up"],
             ),
             mock.call(
                 [
@@ -1236,18 +1162,6 @@ class TestEphemeralIPV4Network(CiTestCase):
             mock.call(
                 ["ip", "-4", "route", "del", "192.168.2.1/32", "dev", "eth0"],
             ),
-            mock.call(
-                [
-                    "ip",
-                    "-family",
-                    "inet",
-                    "link",
-                    "set",
-                    "dev",
-                    "eth0",
-                    "down",
-                ],
-            ),
             mock.call(
                 [
                     "ip",
@@ -1280,44 +1194,6 @@ class TestEphemeralIPV6Network:
             assert expected_setup_calls == m_subp.call_args_list
 
 
-class TestHasURLConnectivity(ResponsesTestCase):
-    def setUp(self):
-        super(TestHasURLConnectivity, self).setUp()
-        self.url = "http://fake/"
-        self.kwargs = {"allow_redirects": True, "timeout": 5.0}
-
-    @mock.patch("cloudinit.net.readurl")
-    def test_url_timeout_on_connectivity_check(self, m_readurl):
-        """A timeout of 5 seconds is provided when reading a url."""
-        self.assertTrue(
-            net.has_url_connectivity({"url": self.url}),
-            "Expected True on url connect",
-        )
-
-    def test_true_on_url_connectivity_success(self):
-        self.responses.add(responses.GET, self.url)
-        self.assertTrue(
-            net.has_url_connectivity({"url": self.url}),
-            "Expected True on url connect",
-        )
-
-    @mock.patch("requests.Session.request")
-    def test_true_on_url_connectivity_timeout(self, m_request):
-        """A timeout raised accessing the url will return False."""
-        m_request.side_effect = requests.Timeout("Fake Connection Timeout")
-        self.assertFalse(
-            net.has_url_connectivity({"url": self.url}),
-            "Expected False on url timeout",
-        )
-
-    def test_true_on_url_connectivity_failure(self):
-        self.responses.add(responses.GET, self.url, body=b"", status=404)
-        self.assertFalse(
-            net.has_url_connectivity({"url": self.url}),
-            "Expected False on url fail",
-        )
-
-
 def _mk_v1_phys(mac, name, driver, device_id):
     v1_cfg = {"type": "physical", "name": name, "mac_address": mac}
     params = {}
@@ -1894,3 +1770,29 @@ class TestIsIpNetwork:
     )
     def test_is_ip_network(self, func, arg, expected_return):
         assert func(arg) == expected_return
+
+
+class TestIsIpInSubnet:
+    """Tests for net.is_ip_in_subnet()."""
+
+    @pytest.mark.parametrize(
+        "func,ip,subnet,expected_return",
+        (
+            (net.is_ip_in_subnet, "192.168.1.1", "2001:67c::1/64", False),
+            (net.is_ip_in_subnet, "2001:67c::1", "192.168.1.1/24", False),
+            (net.is_ip_in_subnet, "192.168.1.1", "192.168.1.1/24", True),
+            (net.is_ip_in_subnet, "192.168.1.1", "192.168.1.1/32", True),
+            (net.is_ip_in_subnet, "192.168.1.2", "192.168.1.1/24", True),
+            (net.is_ip_in_subnet, "192.168.1.2", "192.168.1.1/32", False),
+            (net.is_ip_in_subnet, "192.168.2.2", "192.168.1.1/24", False),
+            (net.is_ip_in_subnet, "192.168.2.2", "192.168.1.1/32", False),
+            (net.is_ip_in_subnet, "2001:67c1::1", "2001:67c1::1/64", True),
+            (net.is_ip_in_subnet, "2001:67c1::1", "2001:67c1::1/128", True),
+            (net.is_ip_in_subnet, "2001:67c1::2", "2001:67c1::1/64", True),
+            (net.is_ip_in_subnet, "2001:67c1::2", "2001:67c1::1/128", False),
+            (net.is_ip_in_subnet, "2002:67c1::1", "2001:67c1::1/8", True),
+            (net.is_ip_in_subnet, "2002:67c1::1", "2001:67c1::1/16", False),
+        ),
+    )
+    def test_is_ip_in_subnet(self, func, ip, subnet, expected_return):
+        assert func(ip, subnet) == expected_return
