@@ -22,6 +22,7 @@ class TestGetFeatures:
             DEPRECATION_INFO_BOUNDARY="devel",
             NOCLOUD_SEED_URL_APPEND_FORWARD_SLASH=False,
             APT_DEB822_SOURCE_LIST_FILE=True,
+            MANUAL_NETWORK_WAIT=False,
         ):
             assert {
                 "ERROR_ON_USER_DATA_FAILURE": True,
@@ -31,4 +32,5 @@ class TestGetFeatures:
                 "NOCLOUD_SEED_URL_APPEND_FORWARD_SLASH": False,
                 "APT_DEB822_SOURCE_LIST_FILE": True,
                 "DEPRECATION_INFO_BOUNDARY": "devel",
+                "MANUAL_NETWORK_WAIT": False,
             } == features.get_features()
